package com.ruoyi.bst.booth.mapper;

import java.util.List;
import com.ruoyi.bst.booth.domain.Booth;
import com.ruoyi.bst.booth.domain.BoothVO;
import com.ruoyi.bst.booth.domain.BoothQuery;
import org.apache.ibatis.annotations.Param;

/**
 * 卡座Mapper接口
 *
 * @author ruoyi
 * @date 2025-04-18
 */
public interface BoothMapper
{
    /**
     * 查询卡座
     *
     * @param boothId 卡座主键
     * @return 卡座
     */
    BoothVO selectBoothByBoothId(Long boothId);

    /**
     * 查询卡座列表
     *
     * @param query 卡座
     * @return 卡座集合
     */
    List<BoothVO> selectBoothList(@Param("query")BoothQuery query);

    /**
     * 新增卡座
     *
     * @param booth 卡座
     * @return 结果
     */
    int insertBooth(Booth booth);

    /**
     * 批量新增卡座
     */
    int batchInsert(@Param("list") List<? extends Booth> list);

    /**
     * 批量修改卡座
     */
    int batchUpdate(@Param("list") List<? extends Booth> list);

    /**
     * 修改卡座
     *
     * @param booth 卡座
     * @return 结果
     */
    public int updateBooth(@Param("data") Booth booth);

    /**
     * 删除卡座
     *
     * @param boothId 卡座主键
     * @return 结果
     */
    int deleteBoothByBoothId(Long boothId);

    /**
     * 批量删除卡座
     *
     * @param boothIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteBoothByBoothIds(@Param("ids") List<Long> boothIds);

    List<Long> selectIdByQuery(@Param("query") BoothQuery query);

    int selectCount(@Param("query") BoothQuery query);

    List<BoothVO> selectBoothByUserId(@Param("userId") Long userId);

    BoothVO selectBoothByBoothNo(@Param("boothNo") String boothNo);
}
